@@ -37,6 +37,10 @@
 #define timespec_ms(tsp) \
 	((tsp)->tv_sec * 1000 + ((tsp)->tv_nsec + 999999) / 1000000)
 
+/* Convert to coarse microseconds with round up */
+#define timespec_us(tsp) \
+	((tsp)->tv_sec * 1000000 + ((tsp)->tv_nsec + 999) / 1000)
+
 /* Operations on timespecs */
 #define timespecclear(tvp)      ((tvp)->tv_sec = (tvp)->tv_nsec = 0)
 #define timespecisset(tvp)      ((tvp)->tv_sec || (tvp)->tv_nsec)
