@@ -178,6 +178,8 @@ typedef struct svc_init_params {
 #define SVC_XPRT_FLAG_UREG		0x0080
 #define SVC_XPRT_TREE_LOCKED		0x0100
 #define SVC_XPRT_FLAG_REMOTE_ADDR_SET	0x0200	/* remote addr was final set */
+#define SVC_XPRT_FLAG_READY		0x0400	/* ready to use */
+#define SVC_XPRT_FLAG_IOQ_WRITING	0x0800	/* xprt is used by svc_ioq_write */
 
 #define SVC_XPRT_FLAG_DESTROYED (SVC_XPRT_FLAG_DESTROYING \
 				| SVC_XPRT_FLAG_RELEASING)
@@ -292,6 +294,11 @@ struct svc_xprt {
 	struct rpc_address xp_local;	/* local address, length, port */
 	struct rpc_address xp_remote;	/* remote address, length, port */
 	struct rpc_address xp_proxy;	/* proxy address, length, port */
+	struct proxy_protocol_tlv_headers proxy_protocol_tlv_headers;
+	/* Network id parsed from proxy protocol.
+	 * Populated only when PARSE_COMMON_NETWORK_ID flag is on
+	 */
+	struct network_id xp_remote_network_id;
 
 #if defined(HAVE_BLKIN)
 	/* blkin tracing */
@@ -311,6 +318,8 @@ struct svc_xprt {
 
 	int32_t xp_refcnt;	/* handle reference count */
 	uint16_t xp_flags;	/* flags */
+	uint32_t xp_unique_id;
+	bool recv_rearm_allowed;
 
 	union {
 		struct in_pktinfo in;
@@ -404,6 +413,10 @@ struct svc_req {
 
 extern void svc_resume(struct svc_req *req);
 
+extern void svc_rqst_qos_suspend_socket(struct svc_xprt *xprt);
+
+extern void svc_rqst_qos_resume_socket(struct svc_xprt *xprt);
+
 /*
  * Ganesha.  Get connected transport type.
  */
@@ -531,7 +544,7 @@ static inline void svc_destroy_it(SVCXPRT *xprt,
 
 	XPRT_TRACE(xprt, __func__, tag, line);
 
-	XPRT_AUTO_TRACEPOINT(xprt, destroy_it, TRACE_DEBUG, "Destroy XPRT");
+	XPRT_AUTO_TRACEPOINT(xprt, destroy_it, TRACE_INFO, "Destroy XPRT");
 
 	if (flags & SVC_XPRT_FLAG_DESTROYING) {
 		/* previously set, do nothing */
@@ -547,22 +560,27 @@ static inline void svc_destroy_it(SVCXPRT *xprt,
 	 * initialization to be done
 	 */
 	retry = 0;
-	while (xprt->xp_ops == NULL && retry < SVC_DESTROY_RETRY) {
+	while (!(xprt->xp_flags & SVC_XPRT_FLAG_READY)
+	    && (retry < SVC_DESTROY_RETRY)) {
 		sched_yield();
 		retry += 1;
 	};
 
 	/* unlink before dropping last ref */
-	(*(xprt)->xp_ops->xp_unlink)(xprt, flags, tag, line);
+	if ((xprt)->xp_ops->xp_unlink)
+		(*(xprt)->xp_ops->xp_unlink)(xprt, flags, tag, line);
 
 	/* Remove references: of xprt from user-data; of user-data from xprt */
 	if ((xprt)->xp_ops->xp_unref_user_data) {
 		(*(xprt)->xp_ops->xp_unref_user_data)(xprt);
 	}
 
-	/* Let's shutdown the sockets so that FIN-ACK could be sent to the
-	 * client immediately. */
-	if (xprt->xp_fd != RPC_ANYFD) {
+	/* Check if the connection was already set as dead or closed,
+	 * If set, let's cleanup and close the FDs, so that FIN-ACK
+	 * could be sent to the client immediately.
+	 * Also for UDP xprt, this is never set, needn't enter here */
+	if ((atomic_fetch_uint16_t(&xprt->xp_flags) & SVC_XPRT_FLAG_CLOSE)
+	    && xprt->xp_fd != RPC_ANYFD) {
 		(void)shutdown(xprt->xp_fd, SHUT_RDWR);
 		if (xprt->xp_fd_send != RPC_ANYFD)
 			(void)shutdown(xprt->xp_fd_send, SHUT_RDWR);
