@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2009, Sun Microsystems, Inc.
+ * Copyright (c) 2012-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,7 +29,7 @@
 /*
  * Copyright (c) 1986-1991 by Sun Microsystems Inc.
  */
-#include <config.h>
+#include "config.h"
 #include <sys/cdefs.h>
 
 /*
@@ -161,7 +162,10 @@ __rpc_getbroadifs(int af, int proto, int socktype, broadlist_t *list)
 #ifdef INET6
 		if (af == AF_INET6 && (ifap->ifa_flags & IFF_MULTICAST)) {
 			sin6 = (struct sockaddr_in6 *)(void *)&bip->broadaddr;
-			inet_pton(af, RPCB_MULTICAST_ADDR, &sin6->sin6_addr);
+			if (inet_pton(af, RPCB_MULTICAST_ADDR, &sin6->sin6_addr)
+			    != 1) {
+				continue;
+			}
 			sin6->sin6_family = af;
 			sin6->sin6_port = ((struct sockaddr_in6 *)
 					   (void *)res->ai_addr)->sin6_port;
@@ -220,9 +224,9 @@ rpc_broadcast_exp(rpcprog_t prog,	/* program number */
 		  rpcvers_t vers,	/* version number */
 		  rpcproc_t proc,	/* procedure number */
 		  xdrproc_t xargs,	/* xdr routine for args */
-		  caddr_t argsp,	/* pointer to args */
+		  void *argsp,	/* pointer to args */
 		  xdrproc_t xresults,	/* xdr routine for results */
-		  caddr_t resultsp,	/* pointer to results */
+		  void *resultsp,	/* pointer to results */
 		  resultproc_t eachresult, /* call with each result obtained */
 		  int inittime,	/* how long to wait initially */
 		  int waittime,	/* maximum time to wait */
@@ -262,7 +266,6 @@ rpc_broadcast_exp(rpcprog_t prog,	/* program number */
 	int i;
 #ifdef PORTMAP
 	size_t outlen_pmap = 0;
-	u_long port;		/* Remote port number */
 	int pmap_flag = 0;	/* UDP exists ? */
 	char *outbuf_pmap = NULL;
 	struct rmtcallargs barg_pmap;	/* Remote arguments */
@@ -401,7 +404,6 @@ rpc_broadcast_exp(rpcprog_t prog,	/* program number */
 		barg_pmap.proc = proc;
 		barg_pmap.args_ptr = argsp;
 		barg_pmap.xdr_args = xargs;
-		bres_pmap.port_ptr = &port;
 		bres_pmap.xdr_results = xresults;
 		bres_pmap.results_ptr = resultsp;
 		xdrmem_create(xdrs, outbuf_pmap, udpbufsz, XDR_ENCODE);
@@ -564,7 +566,7 @@ rpc_broadcast_exp(rpcprog_t prog,	/* program number */
 				pmap_reply_flag = 0;
 				msg.RPCM_ack.ar_verf = _null_auth;
 				msg.RPCM_ack.ar_results.where =
-				    (caddr_t) (void *)&bres;
+				    &bres;
 				msg.RPCM_ack.ar_results.proc =
 				    (xdrproc_t) xdr_rpcb_rmtcallres;
 #ifdef PORTMAP
@@ -574,7 +576,7 @@ rpc_broadcast_exp(rpcprog_t prog,	/* program number */
 				pmap_reply_flag = 1;
 				msg.RPCM_ack.ar_verf = _null_auth;
 				msg.RPCM_ack.ar_results.where =
-				    (caddr_t) (void *)&bres_pmap;
+				    &bres_pmap;
 				msg.RPCM_ack.ar_results.proc =
 				    (xdrproc_t) xdr_rmtcallres;
 #endif				/* PORTMAP */
@@ -593,7 +595,7 @@ rpc_broadcast_exp(rpcprog_t prog,	/* program number */
 						memcpy(&sin, &fdlist[i].raddr,
 						       sizeof(sin));
 						sin.sin_port =
-						    htons((u_short) port);
+						    htons(bres_pmap.port);
 						memcpy(&fdlist[i].raddr, &sin,
 						       sizeof(sin));
 						taddr.len = taddr.maxlen =
@@ -664,9 +666,9 @@ rpc_broadcast(rpcprog_t prog,	/* program number */
 	      rpcvers_t vers,	/* version number */
 	      rpcproc_t proc,	/* procedure number */
 	      xdrproc_t xargs,	/* xdr routine for args */
-	      caddr_t argsp,	/* pointer to args */
+	      void *argsp,	/* pointer to args */
 	      xdrproc_t xresults,	/* xdr routine for results */
-	      caddr_t resultsp,	/* pointer to results */
+	      void *resultsp,	/* pointer to results */
 	      resultproc_t eachresult,	/* call with each result obtained */
 	      const char *nettype /* transport type */)
 {
