@@ -33,6 +33,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <assert.h>
 
 #include <rpc/types.h>
 #include <misc/portable.h>
@@ -76,6 +77,23 @@
 #define SVC_RQST_LOCKED		0x01000000
 #define SVC_RQST_UNLOCK		0x02000000
 
+/* svc_epoll_data is used as the data for svc fd event in epoll.
+ * It enables a direct access to the fd and unique_id, and to
+ * serialize/deserialize the struct into the event format.
+ */
+union svc_epoll_data {
+	struct {
+		int fd;
+		uint32_t unique_id;
+	} data;
+	uint64_t serialized_data;
+};
+
+static_assert(sizeof(union svc_epoll_data) == sizeof(union epoll_data),
+		"incorrect size for svc_epoll_data");
+
+#define SVC_NO_XPRT_UNIQUE_ID	0
+
 static uint32_t round_robin;
 /*static*/ uint32_t wakeups;
 
@@ -154,17 +172,26 @@ void svc_rqst_rec_destroy(struct svc_rqst_rec *sr_rec)
 #endif
 
 	if (sr_rec->sv[0] >= 0) {
+		__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
+			"%s: sr_rec %p sv[0]=%d close",
+			__func__, sr_rec, sr_rec->sv[0]);
 		close(sr_rec->sv[0]);
 		sr_rec->sv[0] = -1;
 	}
 
 	if (sr_rec->sv[1] >= 0) {
+		__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
+			"%s: sr_rec %p sv[1]=%d close",
+			__func__, sr_rec, sr_rec->sv[1]);
 		close(sr_rec->sv[1]);
 		sr_rec->sv[1] = -1;
 	}
 
 #if defined(TIRPC_EPOLL)
 	if (sr_rec->ev_u.epoll.epoll_fd > 0) {
+		__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
+			"%s: sr_rec %p epoll_fd=%d close",
+			__func__, sr_rec, sr_rec->ev_u.epoll.epoll_fd);
 		close(sr_rec->ev_u.epoll.epoll_fd);
 		sr_rec->ev_u.epoll.epoll_fd = -1;
 	}
@@ -385,6 +412,7 @@ svc_rqst_new_evchan(uint32_t *chan_id /* OUT */, void *u_data, uint32_t flags)
 	int code = 0, i;
 	work_pool_fun_t fun = NULL;
 	int32_t ref_rec;
+	union svc_epoll_data ev_data;
 
 	mutex_lock(&svc_rqst_set.mtx);
 	if (!svc_rqst_set.next_id) {
@@ -457,7 +485,10 @@ svc_rqst_new_evchan(uint32_t *chan_id /* OUT */, void *u_data, uint32_t flags)
 		/* permit wakeup of threads blocked in epoll_wait, with a
 		 * couple of possible semantics */
 		sr_rec->ev_u.epoll.ctrl_ev.events = EPOLLIN | EPOLLRDHUP;
-		sr_rec->ev_u.epoll.ctrl_ev.data.fd = sr_rec->sv[1];
+		ev_data.data.fd = sr_rec->sv[1];
+		/* channel event has no xprt unique id */
+		ev_data.data.unique_id = SVC_NO_XPRT_UNIQUE_ID;
+		sr_rec->ev_u.epoll.ctrl_ev.data.u64 = ev_data.serialized_data;
 		code = epoll_ctl(sr_rec->ev_u.epoll.epoll_fd, EPOLL_CTL_ADD,
 				 sr_rec->sv[1], &sr_rec->ev_u.epoll.ctrl_ev);
 		if (code == -1) {
@@ -560,7 +591,7 @@ svc_rqst_unhook_events(struct rpc_dplx_rec *rec, struct svc_rqst_rec *sr_rec,
 				__warnx(TIRPC_DEBUG_FLAG_WARN,
 					"%s: %p fd %d xp_refcnt %" PRId32
 					" sr_rec %p evchan %d ev_refcnt %" PRId32
-					" epoll_fd %d control fd pair (%d:%d) unhook failed (%d)",
+					" epoll_fd %d control fd pair (%d:%d) unhook recv failed (%d)",
 					__func__, rec, rec->xprt.xp_fd,
 					rec->xprt.xp_refcnt,
 					sr_rec, sr_rec->id_k, sr_rec->ev_refcnt,
@@ -571,7 +602,7 @@ svc_rqst_unhook_events(struct rpc_dplx_rec *rec, struct svc_rqst_rec *sr_rec,
 					TIRPC_DEBUG_FLAG_REFCNT,
 					"%s: %p fd %d xp_refcnt %" PRId32
 					" sr_rec %p evchan %d ev_refcnt %" PRId32
-					" epoll_fd %d control fd pair (%d:%d) unhook event %p",
+					" epoll_fd %d control fd pair (%d:%d) unhook recv event %p",
 					__func__, rec, rec->xprt.xp_fd,
 					rec->xprt.xp_refcnt,
 					sr_rec, sr_rec->id_k, sr_rec->ev_refcnt,
@@ -596,7 +627,7 @@ svc_rqst_unhook_events(struct rpc_dplx_rec *rec, struct svc_rqst_rec *sr_rec,
 				__warnx(TIRPC_DEBUG_FLAG_WARN,
 					"%s: %p fd %d xp_refcnt %" PRId32
 					" sr_rec %p evchan %d ev_refcnt %" PRId32
-					" epoll_fd %d control fd pair (%d:%d) unhook failed (%d)",
+					" epoll_fd %d control fd pair (%d:%d) unhook send failed (%d)",
 					__func__, rec, rec->xprt.xp_fd,
 					rec->xprt.xp_refcnt,
 					sr_rec, sr_rec->id_k, sr_rec->ev_refcnt,
@@ -607,7 +638,7 @@ svc_rqst_unhook_events(struct rpc_dplx_rec *rec, struct svc_rqst_rec *sr_rec,
 					TIRPC_DEBUG_FLAG_REFCNT,
 					"%s: %p fd %d xp_refcnt %" PRId32
 					" sr_rec %p evchan %d ev_refcnt %" PRId32
-					" epoll_fd %d control fd pair (%d:%d) unhook event %p",
+					" epoll_fd %d control fd pair (%d:%d) unhook send event %p",
 					__func__, rec, rec->xprt.xp_fd,
 					rec->xprt.xp_refcnt,
 					sr_rec, sr_rec->id_k, sr_rec->ev_refcnt,
@@ -643,6 +674,14 @@ svc_rqst_rearm_events_locked(SVCXPRT *xprt, uint16_t ev_flags)
 	struct svc_rqst_rec *sr_rec = rec->ev_p;
 	int code = EINVAL;
 
+	if (!xprt->recv_rearm_allowed
+	    && (ev_flags & SVC_XPRT_FLAG_ADDED_RECV)) {
+		__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
+				"Failed to rearm as fd %d is suspended",
+				rec->xprt.xp_fd);
+		return 0;
+	}
+
 	XPRT_AUTO_TRACEPOINT(xprt, rearm, TRACE_DEBUG,
 		"Rearm. ev_flags: {}", ev_flags);
 
@@ -664,8 +703,14 @@ svc_rqst_rearm_events_locked(SVCXPRT *xprt, uint16_t ev_flags)
 		ev_flags & SVC_XPRT_FLAG_UREG        ? " UREG" : "",
 		is_rec_shutdown                      ? "sr_rec->ev_flags SHUTDOWN" : "");
 
-	if (is_xprt_destroyed || is_rec_shutdown)
-		return (0);
+	if (is_xprt_destroyed || is_rec_shutdown) {
+		__warnx(TIRPC_DEBUG_FLAG_WARN,
+			"%s: %p fd %d, xp_flags %x, xp_refcnt %" PRId32
+			"is xprt destroyed = %d, is_rec_shutdown = %d",
+			__func__, rec, rec->xprt.xp_fd, rec->xprt.xp_flags,
+			rec->xprt.xp_refcnt, is_xprt_destroyed, is_rec_shutdown);
+		return (1);
+	}
 
 	/* Don't take a ref on the xprt.  We take a ref in hook, and release it
 	 * in unhook. */
@@ -702,7 +747,6 @@ svc_rqst_rearm_events_locked(SVCXPRT *xprt, uint16_t ev_flags)
 					sr_rec, sr_rec->id_k, sr_rec->ev_refcnt,
 					sr_rec->ev_u.epoll.epoll_fd,
 					sr_rec->sv[0], sr_rec->sv[1], code);
-				SVC_RELEASE(xprt, SVC_RELEASE_FLAG_NONE);
 			} else {
 				__warnx(TIRPC_DEBUG_FLAG_SVC_RQST |
 					TIRPC_DEBUG_FLAG_REFCNT,
@@ -742,7 +786,6 @@ svc_rqst_rearm_events_locked(SVCXPRT *xprt, uint16_t ev_flags)
 					sr_rec, sr_rec->id_k, sr_rec->ev_refcnt,
 					sr_rec->ev_u.epoll.epoll_fd,
 					sr_rec->sv[0], sr_rec->sv[1], code);
-				SVC_RELEASE(xprt, SVC_RELEASE_FLAG_NONE);
 			} else {
 				__warnx(TIRPC_DEBUG_FLAG_SVC_RQST |
 					TIRPC_DEBUG_FLAG_REFCNT,
@@ -776,6 +819,9 @@ svc_rqst_hook_events(struct rpc_dplx_rec *rec, struct svc_rqst_rec *sr_rec,
 		     uint16_t ev_flags)
 {
 	int code = EINVAL;
+	const union svc_epoll_data ev_data = {
+			.data.fd = rec->xprt.xp_fd,
+			.data.unique_id = rec->xprt.xp_unique_id};
 
 	XPRT_AUTO_TRACEPOINT(&rec->xprt, hook, TRACE_DEBUG,
 		"Hook. ev_flags: {}", ev_flags);
@@ -812,7 +858,7 @@ svc_rqst_hook_events(struct rpc_dplx_rec *rec, struct svc_rqst_rec *sr_rec,
 			ev = &rec->ev_u.epoll.event_recv;
 
 			/* set up epoll user data */
-			ev->data.fd = rec->xprt.xp_fd;
+			ev->data.u64 = ev_data.serialized_data;
 
 			/* wait for read events, level triggered, oneshot */
 			ev->events = EPOLLONESHOT | EPOLLIN;
@@ -852,7 +898,7 @@ svc_rqst_hook_events(struct rpc_dplx_rec *rec, struct svc_rqst_rec *sr_rec,
 			ev = &rec->ev_u.epoll.event_send;
 
 			/* set up epoll user data.  Lookup needs the primary FD */
-			ev->data.fd = rec->xprt.xp_fd;
+			ev->data.u64 = ev_data.serialized_data;
 
 			/* wait for write events, edge triggered, oneshot */
 			ev->events = EPOLLONESHOT | EPOLLOUT | EPOLLET;
@@ -1127,6 +1173,51 @@ svc_rqst_xprt_register(SVCXPRT *newxprt, SVCXPRT *xprt)
 	return svc_rqst_evchan_reg(sr_rec->id_k, newxprt, SVC_RQST_FLAG_NONE);
 }
 
+
+/*
+ *  need to clear these requests as they are waiting for epoll, which will
+ *   never be triggered after destroy as it unregisters all events
+ */
+static void clear_requests(struct rpc_dplx_rec *rec) {
+	struct poolq_entry *have;
+	struct xdr_ioq *xioq;
+	struct timespec ts = {
+		.tv_sec = 0,
+		.tv_nsec = 0,
+	};
+
+	while (atomic_postset_uint16_t_bits(&rec->xprt.xp_flags,
+			SVC_XPRT_FLAG_IOQ_WRITING)
+	       & SVC_XPRT_FLAG_IOQ_WRITING) {
+		nanosleep(&ts, NULL);
+		XPRT_UNIQUE_AUTO_TRACEPOINT(&rec->xprt, ioq_clearing, TRACE_INFO,
+			"xprt is being transmitted by another thread, can't clear");
+	}
+
+	int release_count = 0;
+	mutex_lock(&rec->writeq.qmutex);
+	have = TAILQ_FIRST(&rec->writeq.qh);
+	while(have != NULL) {
+		xioq = _IOQ(have);
+		while (atomic_postset_uint16_t_bits(&(xioq->ioq_s.qflags),
+						    IOQ_FLAG_WORKING)
+		       & IOQ_FLAG_WORKING) {
+			nanosleep(&ts, NULL);
+			/* We can't clear request when it is in work pool */
+		}
+
+		release_count++;
+		TAILQ_REMOVE(&rec->writeq.qh, have, q);
+		XDR_DESTROY(xioq->xdrs);
+		have = TAILQ_FIRST(&rec->writeq.qh);
+	}
+	mutex_unlock(&rec->writeq.qmutex);
+
+	for(int i = 0; i < release_count; i++) {
+		SVC_RELEASE(&rec->xprt, SVC_RELEASE_FLAG_NONE);
+	}
+}
+
 /*
  * flags indicate locking state
  *
@@ -1156,6 +1247,7 @@ svc_rqst_xprt_unregister(SVCXPRT *xprt, uint32_t flags)
 		return;
 	}
 	svc_rqst_unreg(rec, sr_rec);
+	clear_requests(rec);
 }
 
 #if defined(USE_RPC_RDMA)
@@ -1289,15 +1381,7 @@ svc_rqst_xprt_task_send(struct work_pool_entry *wpe)
 	XPRT_AUTO_TRACEPOINT(&rec->xprt, task_send, TRACE_DEBUG,
 		"Task send");
 
-	/* atomic barrier (above) should protect following values */
-	if (rec->xprt.xp_refcnt > 1
-	    && !(rec->xprt.xp_flags & SVC_XPRT_FLAG_DESTROYED)) {
-		/* (idempotent) xp_flags and xp_refcnt are set atomic.
-		 * xp_refcnt need more than 1 (this task).
-		 */
-		svc_ioq_write(&rec->xprt);
-	}
-
+	svc_ioq_write(&rec->xprt);
 	SVC_RELEASE(&rec->xprt, SVC_RELEASE_FLAG_NONE);
 }
 
@@ -1322,7 +1406,9 @@ svc_rqst_clean_func(SVCXPRT *xprt, void *arg)
 	if (xprt->xp_flags & (SVC_XPRT_FLAG_DESTROYED | SVC_XPRT_FLAG_UREG))
 		return (false);
 
-	if ((acc->ts.tv_sec - REC_XPRT(xprt)->recv.ts.tv_sec) < acc->timeout)
+	/* Make sure recv.ts is initialized */
+	if (REC_XPRT(xprt)->recv.ts.tv_sec &&
+	    ((acc->ts.tv_sec - REC_XPRT(xprt)->recv.ts.tv_sec) < acc->timeout))
 		return (false);
 
 	SVC_DESTROY(xprt);
@@ -1378,8 +1464,9 @@ svc_rqst_epoll_event(struct svc_rqst_rec *sr_rec, struct epoll_event *ev)
 	uint16_t xp_flags, ev_flag = 0;
 	struct xdr_ioq *ioq = NULL;
 	work_pool_fun_t fun;
+	const union svc_epoll_data ev_data = {.serialized_data = ev->data.u64};
 
-	if (unlikely(ev->data.fd == sr_rec->sv[1])) {
+	if (unlikely(ev_data.data.fd == sr_rec->sv[1])) {
 		/* signalled -- there was a wakeup on ctrl_ev (see
 		 * top-of-loop) */
 		__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
@@ -1394,14 +1481,31 @@ svc_rqst_epoll_event(struct svc_rqst_rec *sr_rec, struct epoll_event *ev)
 		return (NULL);
 	}
 
-	xprt = svc_xprt_lookup(ev->data.fd, NULL);
+	xprt = svc_xprt_lookup(ev_data.data.fd, NULL);
 	if (!xprt) {
+		NTIRPC_AUTO_TRACEPOINT(xprt, epoll_fail_lookup, TRACE_INFO,
+				"no xprt found fd = {}", ev_data.data.fd);
 		__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
 			"%s: fd %d no associated xprt",
-			__func__, ev->data.fd);
+			__func__, ev_data.data.fd);
 		return (NULL);
 	}
 	/* At this point, we have a ref on the xprt, and know it's valid */
+	/* verify that indeed this is the xprt as of the event. */
+	if (ev_data.data.unique_id != xprt->xp_unique_id) {
+		NTIRPC_AUTO_TRACEPOINT(xprt, unique_id_mismatch, TRACE_INFO,
+				"xprt for fd = {} found with unique_id = {}, but event unique_id = {}",
+				ev_data.data.fd, xprt->xp_unique_id,
+				ev_data.data.unique_id);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: event fd %d with xp_unique_id %" PRIu32
+			" differs from of xp_unique_id %" PRIu32 " of xprt %p. Release and ignore",
+			__func__, ev_data.data.fd, ev_data.data.unique_id,
+			xprt->xp_unique_id, xprt);
+		SVC_RELEASE(xprt, SVC_RELEASE_FLAG_NONE);
+		return (NULL);
+	}
+
 	rec = REC_XPRT(xprt);
 
 	__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
@@ -1422,6 +1526,8 @@ svc_rqst_epoll_event(struct svc_rqst_rec *sr_rec, struct epoll_event *ev)
 		ioq = rec->ev_u.epoll.xioq_send;
 		fun = svc_rqst_xprt_task_send;
 	} else {
+		XPRT_UNIQUE_AUTO_TRACEPOINT(&rec->xprt, epoll_event, TRACE_INFO,
+			"Unhandled epoll event. ev_flag: {}", ev->events);
 		/* This is some other event... */
 		SVC_RELEASE(&rec->xprt, SVC_RELEASE_FLAG_NONE);
 		return NULL;
@@ -1443,7 +1549,7 @@ svc_rqst_epoll_event(struct svc_rqst_rec *sr_rec, struct epoll_event *ev)
 		ev_flag & SVC_XPRT_FLAG_ADDED_RECV ? " ADDED_RECV" : "",
 		ev_flag & SVC_XPRT_FLAG_ADDED_SEND ? " ADDED_SEND" : "");
 
-	XPRT_AUTO_TRACEPOINT(&rec->xprt, epoll_event, TRACE_DEBUG,
+	XPRT_UNIQUE_AUTO_TRACEPOINT(&rec->xprt, epoll_event, TRACE_DEBUG,
 		"Epoll event. ev_flag: {}", ev_flag);
 
 	if (rec->xprt.xp_refcnt > 1
@@ -1459,10 +1565,16 @@ svc_rqst_epoll_event(struct svc_rqst_rec *sr_rec, struct epoll_event *ev)
 		ioq->rec = rec;
 		return ioq;
 	}
-
+	XPRT_UNIQUE_AUTO_TRACEPOINT(&rec->xprt, epoll_event, TRACE_INFO,
+		"irrelevant epoll event. ev_flag: {}", ev_flag);
 	/* Do not return destroyed transports.
 	 * Probably log non-fatal "WARNING! already destroying!"
 	 */
+	__warnx(TIRPC_DEBUG_FLAG_ERROR,
+		"%s: %p fd %d already destroying. prev flags %x, curr flags %x, ev_flag %x,"
+		"ref_count = %" PRId32,
+		__func__, &rec->xprt, rec->xprt.xp_fd, xp_flags, xprt->xp_flags,
+		ev_flag, xprt->xp_refcnt);
 	SVC_RELEASE(&rec->xprt, SVC_RELEASE_FLAG_NONE);
 	return (NULL);
 }
@@ -1695,6 +1807,22 @@ svc_rqst_delete_evchan(uint32_t chan_id)
 	return (code);
 }
 
+void svc_rqst_qos_suspend_socket(struct svc_xprt *xprt)
+{
+	xprt->recv_rearm_allowed = false;
+}
+
+void svc_rqst_qos_resume_socket(struct svc_xprt *xprt)
+{
+	xprt->recv_rearm_allowed = true;
+	if (unlikely(svc_rqst_rearm_events(xprt, SVC_XPRT_FLAG_ADDED_RECV))) {
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+				"%s: %p fd %d svc_rqst_rearm_events failed",
+				__func__, xprt, xprt->xp_fd);
+	}
+	SVC_STAT(xprt);
+}
+
 void
 svc_rqst_shutdown(void)
 {
